//	This is the Planets.h header file.  It contains all of the information as well as
//	classes for the manipulation of data in a PDATAx.dat file which is used in VGA Planets
//	for the planet data.
#ifndef FSTREAM_H
#define FSTREAM_H

#include <fstream.h>
#endif


#ifndef PLANETS_H
#define PLANETS_H

/*  I use the typedefs below to simulate the type sizes that are used in the files.
	Tim Wisseman programmed the game in BASIC which uses the data types WORD, DWORD, and BYTE.
	In C++ the size of the int variable type changes with the type of system.  It is 2 bytes on
	a 16 bit system and 4 bytes on a 32 bit system.  I noticed that the thyps short and long
	are not dynamic in this way and so I used them.  The typedefs below are only so that a BASIC
	programmer would know what was goin on. */
typedef short word;
typedef long  dword;

/*	The following class is used to load and manipulate the data in the planet files.  As a note,
	the variables with the n, t, d, or m, after or before the underscore are Neutronium,
	Tritanium, Duranium, and Molybdenum respectivly */
class VPlanet
{
private:
	word player_id;			// The ID number of the controlling player ( 0 if unowned )
	word planet_id;			// The ID number of the planet
	char FC[3];				// The planet's Friendly Code.  3 bytes
	word mines;				// Number of mines on the planet
	word factories;			// Number of factories
	word defences;			// Number of Defence Posts
	dword mined_n;			// The amount of a Mineral that has been mined already
	dword mined_t;			
	dword mined_d;			
	dword mined_m;			
	dword colonists;		// Number of Colonist clans ( 100 colonists per clan )
	dword supplies;			// Amount of supplies on the planet
	dword megacredits;		// Amount of Megacredits
	dword n_inground;		// Amount of Minerals in the ground
	dword t_inground;		
	dword d_inground;		
	dword m_inground;		
	word n_density;			// The Density of the Mineral
	word t_density;
	word d_density;
	word m_density;
	word colonist_tax;		// The Colonist's tax rate
	word native_tax;		// The Native's tax rate
	word colonist_hp;		// Colonist happy points
	word native_happiness;	// Native happy points
	word native_gov;		// Native government type
	dword natives;			// Number of natives ( 100 natives per 'clan' )
	word native_race;		// Type of native
	word temperature;		// Planet temperature
	word build_base;		// Is a base to be built here?
public:
	//	The following functions are used to retrieve data in the above variables
	int getPlayerID() { return player_id; }
	int getPlanetID() { return planet_id; }
	char* getFC() 
	{ 
		char* FCode = FC;
		return FCode;
	}
	int getMines() { return mines; }
	int getFactories() { return factories; }
	int getDefences() { return defences; }
	long getMinedN() { return mined_n; }
	long getMinedT() { return mined_t; }
	long getMinedD() { return mined_d; }
	long getMinedM() { return mined_m; }
	long getColonists() { return colonists; }
	long getSupplies() { return supplies; }
	long getMegacredits() { return megacredits; }
	long getNInground() { return n_inground; }
	long getTInground() { return t_inground; }
	long getDInground() { return d_inground; }
	long getMInground() { return m_inground; }
	int getNDensity() { return n_density; }
	int getTDensity() { return t_density; }
	int getDDensity() { return d_density; }
	int getMDenstiy() { return m_density; }
	int getColonistTax() { return colonist_tax; }
	int getNativeTax() { return native_tax; }
	int getColonistHP() { return colonist_hp; }
	int getNativeHappiness() { return native_happiness; }
	int getNativeGov() { return native_gov; }
	long getNatives() { return natives; }
	int getNativeRace() { return native_race; }
	int getTemperature() { return temperature; }
	int getBuildBase() { return build_base; }

	//	The following functions are used to set the values of the above variables
	void setPlayerID(short id) { player_id = id; }
	void setPlanetID(short id) { planet_id = id; }
	void setFC(char* FCode) 
	{ 
		char* Code = FC;
		Code = FCode;
	}
	void setMines(short newMines) { mines = newMines; }
	void setFactories(short newFactories) { factories = newFactories; }
	void setDefences(short newDefences) { defences = newDefences; }
	void setMinedN(long MinedN) { mined_n = MinedN; }
	void setMinedT(long MinedT) { mined_t = MinedT; }
	void setMinedD(long MinedD) { mined_d = MinedD; }
	void setMinedM(long MinedM) { mined_m = MinedM; }
	void setColonists(long newColonists) { colonists = newColonists; }
	void setSupplies(long newSupplies) { supplies = newSupplies; }
	void setMegacredits(long newMegacredits) { megacredits = newMegacredits; }
	void setNInground(long newNInground) { n_inground = newNInground; }
	void setTInground(long newTInground) { t_inground = newTInground; }
	void setDInground(long newDInground) { d_inground = newDInground; }
	void setMInground(long newMInground) { m_inground = newMInground; }
	void setNDensity(short newNDensity) { n_density = newNDensity; }
	void setTDensity(short newTDensity) { t_density = newTDensity; }
	void setDDensity(short newDDensity) { d_density = newDDensity; }
	void setMDensity(short newMDensity) { m_density = newMDensity; }
	void setColonistTax(short newTax) { colonist_tax = newTax; }
	void setNativeTax(short newTax) { native_tax = newTax; }
	void setNativeHappiness(short newHappy) { native_happiness = newHappy; }
	void setColonistHP(short newHappy) { colonist_hp = newHappy; }
	void setNativeGov(short newGov) { native_gov = newGov; }
	void setNatives(short newNatives) { natives = newNatives; }
	void setNativeRace(short newRace) { native_race = newRace; }
	void setTemperature(short newTemp) { temperature = newTemp; }
	void setBuildBase(short buildBase) { build_base = buildBase; }

	//	The following functions are used to read and write that data from the PDATAx.dat files
	void readFile(int pos, char* file)
	{
		ifstream infile(file, ios::binary);
		infile.seekg((pos*sizeof(VPlanet))+4);
		infile.read((char*)this, sizeof(*this) );
	}

	void writeFile(int pos, char* file)
	{
		ofstream outfile(file, ios::binary);
		outfile.seekp((pos*sizeof(VPlanet))+4);
		outfile.write((char*)this, sizeof(*this));
	}
};

#endif

char* PDATA1 = "PDATA1.DAT";
char* PDATA2 = "PDATA2.DAT";
char* PDATA3 = "PDATA3.DAT";
char* PDATA4 = "PDATA4.DAT";
char* PDATA5 = "PDATA5.DAT";
char* PDATA6 = "PDATA6.DAT";
char* PDATA7 = "PDATA7.DAT";
char* PDATA8 = "PDATA8.DAT";
char* PDATA9 = "PDATA9.DAT";
char* PDATA10 = "PDATA10.DAT";
char* PDATA11 = "PDATA11.DAT";